@@ -26,16 +26,11 @@
 #include <errno.h>
 #include <string.h>
 
-#include <gdbus.h>
-
 #include "connman.h"
 
-static DBusConnection *connection = NULL;
-
 struct connman_device {
 	struct connman_element element;
 	enum connman_device_type type;
-	enum connman_device_mode mode;
 	connman_bool_t offlinemode;
 	connman_bool_t blocked;
 	connman_bool_t powered;
@@ -45,11 +40,11 @@ struct connman_device {
 	connman_bool_t disconnected;
 	connman_bool_t reconnect;
 	connman_uint16_t scan_interval;
+	connman_uint16_t backoff_interval;
 	char *name;
 	char *node;
 	char *address;
 	char *interface;
-	char *control;
 	char *ident;
 	int phyindex;
 	unsigned int connections;
@@ -58,16 +53,13 @@ struct connman_device {
 	struct connman_device_driver *driver;
 	void *driver_data;
 
-	connman_bool_t registered;
-
 	char *last_network;
 	struct connman_network *network;
 	GHashTable *networks;
-
-	DBusMessage *pending;
-	guint timeout;
 };
 
+#define SCAN_INITIAL_DELAY 10
+
 static gboolean device_scan_trigger(gpointer user_data)
 {
 	struct connman_device *device = user_data;
@@ -98,9 +90,23 @@ static void reset_scan_trigger(struct connman_device *device)
 	clear_scan_trigger(device);
 
 	if (device->scan_interval > 0) {
-		guint interval = device->scan_interval;
+		guint interval;
+
+		if (g_hash_table_size(device->networks) == 0) {
+			if (device->backoff_interval >= device->scan_interval)
+				device->backoff_interval = SCAN_INITIAL_DELAY;
+			interval = device->backoff_interval;
+		} else
+			interval = device->scan_interval;
+
+		DBG("interval %d", interval);
+
 		device->scan_timeout = g_timeout_add_seconds(interval,
 					device_scan_trigger, device);
+
+		device->backoff_interval *= 2;
+		if (device->backoff_interval > device->scan_interval)
+			device->backoff_interval = device->scan_interval;
 	}
 }
 
@@ -135,6 +141,9 @@ static const char *type2description(enum connman_device_type type)
 		return "GPS";
 	case CONNMAN_DEVICE_TYPE_CELLULAR:
 		return "Cellular";
+	case CONNMAN_DEVICE_TYPE_GADGET:
+		return "Gadget";
+
 	}
 
 	return NULL;
@@ -158,6 +167,9 @@ static const char *type2string(enum connman_device_type type)
 		return "gps";
 	case CONNMAN_DEVICE_TYPE_CELLULAR:
 		return "cellular";
+	case CONNMAN_DEVICE_TYPE_GADGET:
+		return "gadget";
+
 	}
 
 	return NULL;
@@ -182,30 +194,20 @@ enum connman_service_type __connman_device_get_service_type(struct connman_devic
 		return CONNMAN_SERVICE_TYPE_BLUETOOTH;
 	case CONNMAN_DEVICE_TYPE_CELLULAR:
 		return CONNMAN_SERVICE_TYPE_CELLULAR;
+	case CONNMAN_DEVICE_TYPE_GADGET:
+		return CONNMAN_SERVICE_TYPE_GADGET;
+
 	}
 
 	return CONNMAN_SERVICE_TYPE_UNKNOWN;
 }
 
-static void powered_changed(struct connman_device *device)
-{
-	connman_dbus_property_changed_basic(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "Powered",
-					DBUS_TYPE_BOOLEAN, &device->powered);
-}
-
-static void blocked_changed(struct connman_device *device)
-{
-	connman_dbus_property_changed_basic(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "Blocked",
-					DBUS_TYPE_BOOLEAN, &device->blocked);
-}
-
 int __connman_device_enable(struct connman_device *device)
 {
 	int err;
+	enum connman_service_type type;
 
-	DBG("device %p", device);
+	DBG("device %p %d", device, device->blocked);
 
 	if (!device->driver || !device->driver->enable)
 		return -EOPNOTSUPP;
@@ -216,17 +218,28 @@ int __connman_device_enable(struct connman_device *device)
 	if (device->blocked == TRUE)
 		return -ENOLINK;
 
+	connman_device_set_disconnected(device, FALSE);
+	device->scanning = FALSE;
+
 	err = device->driver->enable(device);
-	if (err < 0) {
-		if (err == -EINPROGRESS)
+	if (err < 0 && err != -EALREADY) {
+		if (err == -EINPROGRESS) {
 			device->powered_pending = TRUE;
+			device->offlinemode = FALSE;
+			if (__connman_profile_get_offlinemode() == TRUE)
+				__connman_profile_set_offlinemode(FALSE, FALSE);
+		}
 		return err;
 	}
 
 	device->powered_pending = TRUE;
 	device->powered = TRUE;
+	device->offlinemode = FALSE;
+	if (__connman_profile_get_offlinemode() == TRUE)
+		__connman_profile_set_offlinemode(FALSE, FALSE);
 
-	__connman_technology_enable_device(device);
+	type = __connman_device_get_service_type(device);
+	__connman_technology_enable(type);
 
 	return 0;
 }
@@ -234,6 +247,7 @@ int __connman_device_enable(struct connman_device *device)
 int __connman_device_disable(struct connman_device *device)
 {
 	int err;
+	enum connman_service_type type;
 
 	DBG("device %p", device);
 
@@ -253,16 +267,19 @@ int __connman_device_disable(struct connman_device *device)
 	g_hash_table_remove_all(device->networks);
 
 	err = device->driver->disable(device);
-	if (err < 0) {
+	if (err < 0 && err != -EALREADY) {
 		if (err == -EINPROGRESS)
 			device->powered_pending = FALSE;
 		return err;
 	}
 
+	device->connections = 0;
+
 	device->powered_pending = FALSE;
 	device->powered = FALSE;
 
-	__connman_technology_disable_device(device);
+	type = __connman_device_get_service_type(device);
+	__connman_technology_disable(type);
 
 	return 0;
 }
@@ -277,292 +294,12 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 		return __connman_device_disable(device);
 }
 
-void __connman_device_list(DBusMessageIter *iter, void *user_data)
-{
-	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_DEVICE, iter);
-}
-
-static void append_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct connman_element *element = value;
-	DBusMessageIter *iter = user_data;
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-							&element->path);
-}
-
-static void append_networks(DBusMessageIter *iter, void *user_data)
-{
-	struct connman_device *device = user_data;
-
-	g_hash_table_foreach(device->networks, append_path, iter);
-}
-
-static DBusMessage *get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_device *device = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-	const char *str;
-
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_PUBLIC) < 0)
-		return __connman_error_permission_denied(msg);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	connman_dbus_dict_open(&array, &dict);
-
-	if (device->name != NULL)
-		connman_dbus_dict_append_basic(&dict, "Name",
-					DBUS_TYPE_STRING, &device->name);
-
-	str = type2string(device->type);
-	if (str != NULL)
-		connman_dbus_dict_append_basic(&dict, "Type",
-						DBUS_TYPE_STRING, &str);
-
-	if (device->address != NULL)
-		connman_dbus_dict_append_basic(&dict, "Address",
-					DBUS_TYPE_STRING, &device->address);
-
-	if (device->interface != NULL)
-		connman_dbus_dict_append_basic(&dict, "Interface",
-					DBUS_TYPE_STRING, &device->interface);
-
-	connman_dbus_dict_append_basic(&dict, "Powered",
-					DBUS_TYPE_BOOLEAN, &device->powered);
-
-	connman_dbus_dict_append_basic(&dict, "Blocked",
-					DBUS_TYPE_BOOLEAN, &device->blocked);
-
-	if (device->driver && device->driver->scan)
-		connman_dbus_dict_append_basic(&dict, "Scanning",
-					DBUS_TYPE_BOOLEAN, &device->scanning);
-
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-		break;
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		if (device->scan_interval > 0)
-			connman_dbus_dict_append_basic(&dict, "ScanInterval",
-				DBUS_TYPE_UINT16, &device->scan_interval);
-
-		connman_dbus_dict_append_array(&dict, "Networks",
-				DBUS_TYPE_OBJECT_PATH, append_networks, device);
-		break;
-	}
-
-	connman_dbus_dict_close(&array, &dict);
-
-	return reply;
-}
-
-static gboolean powered_timeout(gpointer user_data)
-{
-	struct connman_device *device = user_data;
-
-	DBG("device %p", device);
-
-	device->timeout = 0;
-
-	if (device->pending != NULL) {
-		DBusMessage *reply;
-
-		reply = __connman_error_operation_timeout(device->pending);
-		if (reply != NULL)
-			g_dbus_send_message(connection, reply);
-
-		dbus_message_unref(device->pending);
-		device->pending = NULL;
-	}
-
-	return FALSE;
-}
-
-static DBusMessage *set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_device *device = data;
-	DBusMessageIter iter, value;
-	const char *name;
-	int type;
-
-	DBG("conn %p", conn);
-
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
-		return __connman_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	type = dbus_message_iter_get_arg_type(&value);
-
-	if (g_str_equal(name, "Powered") == TRUE) {
-		connman_bool_t powered;
-		int err;
-
-		if (type != DBUS_TYPE_BOOLEAN)
-			return __connman_error_invalid_arguments(msg);
-
-		dbus_message_iter_get_basic(&value, &powered);
-
-		device->powered_persistent = powered;
-
-		__connman_storage_save_device(device);
-
-		if (device->powered == powered)
-			return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-
-		if (device->pending != NULL)
-			return __connman_error_in_progress(msg);
-
-		err = set_powered(device, powered);
-		if (err < 0) {
-			if (err != -EINPROGRESS)
-				return __connman_error_failed(msg, -err);
-
-			device->pending = dbus_message_ref(msg);
-
-			device->timeout = g_timeout_add_seconds(15,
-						powered_timeout, device);
-
-			return NULL;
-		}
-	} else if (g_str_equal(name, "ScanInterval") == TRUE) {
-		connman_uint16_t interval;
-
-		switch (device->mode) {
-		case CONNMAN_DEVICE_MODE_UNKNOWN:
-		case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-		case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-			break;
-		}
-
-		if (type != DBUS_TYPE_UINT16)
-			return __connman_error_invalid_arguments(msg);
-
-		dbus_message_iter_get_basic(&value, &interval);
-
-		if (device->scan_interval != interval) {
-			device->scan_interval = interval;
-
-			__connman_storage_save_device(device);
-
-			reset_scan_trigger(device);
-		}
-	} else
-		return __connman_error_invalid_property(msg);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *propose_scan(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_device *device = data;
-	int err;
-
-	DBG("conn %p", conn);
-
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-		return __connman_error_not_supported(msg);
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		break;
-	}
-
-	err = __connman_device_scan(device);
-	if (err < 0)
-		return __connman_error_failed(msg, -err);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static GDBusMethodTable device_methods[] = {
-	{ "GetProperties", "",      "a{sv}", get_properties },
-	{ "SetProperty",   "sv",    "",      set_property,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "ProposeScan",   "",      "",      propose_scan   },
-	{ },
-};
-
-static GDBusSignalTable device_signals[] = {
-	{ "PropertyChanged", "sv" },
-	{ },
-};
-
-static int register_interface(struct connman_element *element)
-{
-	struct connman_device *device = element->device;
-
-	DBG("element %p name %s", element, element->name);
-
-	if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_DEVICE_INTERFACE,
-					device_methods, device_signals,
-					NULL, device, NULL) == FALSE) {
-		connman_error("Failed to register %s device", element->path);
-		return -EIO;
-	}
-
-	device->registered = TRUE;
-
-	return 0;
-}
-
-static void unregister_interface(struct connman_element *element)
-{
-	struct connman_device *device = element->device;
-
-	DBG("element %p name %s", element, element->name);
-
-	device->registered = FALSE;
-
-	g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_DEVICE_INTERFACE);
-}
-
 static int setup_device(struct connman_device *device)
 {
-	int err;
-
 	DBG("device %p", device);
 
-	err = register_interface(&device->element);
-	if (err < 0) {
-		if (device->driver->remove)
-			device->driver->remove(device);
-		device->driver = NULL;
-		return err;
-	}
-
 	__connman_technology_add_device(device);
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		break;
-	}
-
-	if (__connman_udev_get_blocked(device->phyindex) == TRUE)
-		return 0;
-
 	if (device->offlinemode == FALSE &&
 				device->powered_persistent == TRUE)
 		__connman_device_enable(device);
@@ -590,6 +327,8 @@ static void probe_driver(struct connman_element *element, gpointer user_data)
 
 	element->device->driver = driver;
 
+	__connman_element_set_driver(element);
+
 	setup_device(element->device);
 }
 
@@ -599,17 +338,8 @@ static void remove_device(struct connman_device *device)
 
 	__connman_device_disable(device);
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		break;
-	}
-
 	__connman_technology_remove_device(device);
 
-	unregister_interface(&device->element);
-
 	if (device->driver->remove)
 		device->driver->remove(device);
 
@@ -634,7 +364,7 @@ connman_bool_t __connman_device_has_driver(struct connman_device *device)
 	if (device == NULL || device->driver == NULL)
 		return FALSE;
 
-	return device->registered;
+	return TRUE;
 }
 
 static GSList *driver_list = NULL;
@@ -692,6 +422,8 @@ static void unregister_network(gpointer data)
 
 	connman_element_unregister((struct connman_element *) network);
 
+	__connman_network_set_device(network, NULL);
+
 	connman_network_unref(network);
 }
 
@@ -701,23 +433,12 @@ static void device_destruct(struct connman_element *element)
 
 	DBG("element %p name %s", element, element->name);
 
-	if (device->timeout > 0) {
-		g_source_remove(device->timeout);
-		device->timeout = 0;
-	}
-
 	clear_scan_trigger(device);
 
-	if (device->pending != NULL) {
-		dbus_message_unref(device->pending);
-		device->pending = NULL;
-	}
-
 	g_free(device->ident);
 	g_free(device->node);
 	g_free(device->name);
 	g_free(device->address);
-	g_free(device->control);
 	g_free(device->interface);
 
 	g_free(device->last_network);
@@ -740,6 +461,7 @@ struct connman_device *connman_device_create(const char *node,
 {
 	struct connman_device *device;
 	const char *str;
+	enum connman_service_type service_type;
 
 	DBG("node %s type %d", node, type);
 
@@ -766,12 +488,15 @@ struct connman_device *connman_device_create(const char *node,
 
 	device->type = type;
 	device->name = g_strdup(type2description(device->type));
-	device->mode = CONNMAN_DEVICE_MODE_UNKNOWN;
 
 	device->powered_persistent = TRUE;
 
 	device->phyindex = -1;
 
+	service_type = __connman_device_get_service_type(device);
+	device->blocked = __connman_technology_get_blocked(service_type);
+	device->backoff_interval = SCAN_INITIAL_DELAY;
+
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
@@ -793,6 +518,9 @@ struct connman_device *connman_device_create(const char *node,
 	case CONNMAN_DEVICE_TYPE_CELLULAR:
 		device->scan_interval = 0;
 		break;
+	case CONNMAN_DEVICE_TYPE_GADGET:
+		device->scan_interval = 0;
+		break;
 	}
 
 	device->networks = g_hash_table_new_full(g_str_hash, g_str_equal,
@@ -842,28 +570,6 @@ enum connman_device_type connman_device_get_type(struct connman_device *device)
 	return device->type;
 }
 
-/**
- * connman_device_get_name:
- * @device: device structure
- *
- * Get unique name of device
- */
-const char *connman_device_get_name(struct connman_device *device)
-{
-	return device->element.name;
-}
-
-/**
- * connman_device_get_path:
- * @device: device structure
- *
- * Get path name of device
- */
-const char *connman_device_get_path(struct connman_device *device)
-{
-	return device->element.path;
-}
-
 /**
  * connman_device_set_index:
  * @device: device structure
@@ -902,12 +608,11 @@ void __connman_device_set_phyindex(struct connman_device *device,
  * connman_device_set_interface:
  * @device: device structure
  * @interface: interface name
- * @control: control interface
  *
  * Set interface name of device
  */
 void connman_device_set_interface(struct connman_device *device,
-				const char *interface, const char *control)
+						const char *interface)
 {
 	g_free(device->element.devname);
 	device->element.devname = g_strdup(interface);
@@ -915,9 +620,6 @@ void connman_device_set_interface(struct connman_device *device,
 	g_free(device->interface);
 	device->interface = g_strdup(interface);
 
-	g_free(device->control);
-	device->control = g_strdup(control);
-
 	if (device->name == NULL) {
 		const char *str = type2description(device->type);
 		if (str != NULL && device->interface != NULL)
@@ -926,11 +628,6 @@ void connman_device_set_interface(struct connman_device *device,
 	}
 }
 
-const char *connman_device_get_control(struct connman_device *device)
-{
-	return device->control;
-}
-
 /**
  * connman_device_set_ident:
  * @device: device structure
@@ -945,35 +642,11 @@ void connman_device_set_ident(struct connman_device *device,
 	device->ident = g_strdup(ident);
 }
 
-const char *__connman_device_get_ident(struct connman_device *device)
+const char *connman_device_get_ident(struct connman_device *device)
 {
 	return device->ident;
 }
 
-/**
- * connman_device_set_mode:
- * @device: device structure
- * @mode: network mode
- *
- * Change network mode of device
- */
-void connman_device_set_mode(struct connman_device *device,
-						enum connman_device_mode mode)
-{
-	device->mode = mode;
-}
-
-/**
- * connman_device_get_mode:
- * @device: device structure
- *
- * Get network mode of device
- */
-enum connman_device_mode connman_device_get_mode(struct connman_device *device)
-{
-	return device->mode;
-}
-
 /**
  * connman_device_set_powered:
  * @device: device structure
@@ -984,55 +657,43 @@ enum connman_device_mode connman_device_get_mode(struct connman_device *device)
 int connman_device_set_powered(struct connman_device *device,
 						connman_bool_t powered)
 {
+	int err;
+	enum connman_service_type type;
+
 	DBG("driver %p powered %d", device, powered);
 
-	if (device->timeout > 0) {
-		g_source_remove(device->timeout);
-		device->timeout = 0;
-	}
-
-	if (device->pending != NULL) {
-		g_dbus_send_reply(connection, device->pending,
-							DBUS_TYPE_INVALID);
-
-		dbus_message_unref(device->pending);
-		device->pending = NULL;
-	}
-
 	if (device->powered == powered) {
 		device->powered_pending = powered;
 		return -EALREADY;
 	}
 
 	if (powered == TRUE)
-		__connman_device_enable(device);
+		err = __connman_device_enable(device);
 	else
-		__connman_device_disable(device);
+		err = __connman_device_disable(device);
+
+	if (err < 0 && err != -EINPROGRESS && err != -EALREADY)
+		return err;
 
 	device->powered = powered;
 	device->powered_pending = powered;
 
+	type = __connman_device_get_service_type(device);
+
 	if (device->powered == TRUE)
-		__connman_technology_enable_device(device);
+		__connman_technology_enable(type);
 	else
-		__connman_technology_disable_device(device);
+		__connman_technology_disable(type);
 
-	if (device->offlinemode == TRUE && powered == TRUE) {
-		powered_changed(device);
+	if (device->offlinemode == TRUE && powered == TRUE)
 		return connman_device_set_powered(device, FALSE);
-	}
-
-	if (device->registered == FALSE)
-		return 0;
-
-	powered_changed(device);
 
 	if (powered == FALSE)
 		return 0;
 
 	reset_scan_trigger(device);
 
-	if (device->driver->scan)
+	if (device->driver && device->driver->scan)
 		device->driver->scan(device);
 
 	return 0;
@@ -1047,8 +708,6 @@ int __connman_device_set_blocked(struct connman_device *device,
 
 	device->blocked = blocked;
 
-	blocked_changed(device);
-
 	if (device->offlinemode == TRUE)
 		return 0;
 
@@ -1093,8 +752,11 @@ int __connman_device_enable_persistent(struct connman_device *device)
 	err = __connman_device_enable(device);
 	if (err == 0 || err == -EINPROGRESS) {
 		device->offlinemode = FALSE;
-		if (__connman_profile_get_offlinemode() == TRUE)
+		if (__connman_profile_get_offlinemode() == TRUE) {
 			__connman_profile_set_offlinemode(FALSE, FALSE);
+
+			__connman_profile_save_default();
+		}
 	}
 
 	return err;
@@ -1125,7 +787,7 @@ int __connman_device_disconnect(struct connman_device *device)
 	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
 		struct connman_network *network = value;
 
-		if (__connman_network_get_connecting(network) == TRUE) {
+		if (connman_network_get_connecting(network) == TRUE) {
 			/*
 			 * Skip network in the process of connecting.
 			 * This is a workaround for WiFi networks serviced
@@ -1147,6 +809,14 @@ int __connman_device_disconnect(struct connman_device *device)
 	return 0;
 }
 
+static void mark_network_available(gpointer key, gpointer value,
+                                                        gpointer user_data)
+{
+	struct connman_network *network = value;
+
+	connman_network_set_available(network, TRUE);
+}
+
 static void mark_network_unavailable(gpointer key, gpointer value,
 							gpointer user_data)
 {
@@ -1178,11 +848,18 @@ void __connman_device_cleanup_networks(struct connman_device *device)
 					remove_unavailable_network, NULL);
 }
 
-static void scanning_changed(struct connman_device *device)
+connman_bool_t __connman_device_scanning(struct connman_device *device)
 {
-	connman_dbus_property_changed_basic(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "Scanning",
-					DBUS_TYPE_BOOLEAN, &device->scanning);
+	return device->scanning;
+}
+
+void connman_device_reset_scanning(struct connman_device *device)
+{
+	device->scanning = FALSE;
+
+	g_hash_table_foreach(device->networks,
+				mark_network_available, NULL);
+
 }
 
 /**
@@ -1205,8 +882,6 @@ int connman_device_set_scanning(struct connman_device *device,
 
 	device->scanning = scanning;
 
-	scanning_changed(device);
-
 	if (scanning == TRUE) {
 		reset_scan_trigger(device);
 
@@ -1221,9 +896,6 @@ int connman_device_set_scanning(struct connman_device *device,
 	if (device->connections > 0)
 		return 0;
 
-	if (device->disconnected == TRUE)
-		return 0;
-
 	__connman_service_auto_connect();
 
 	return 0;
@@ -1241,14 +913,6 @@ int connman_device_set_disconnected(struct connman_device *device,
 {
 	DBG("device %p disconnected %d", device, disconnected);
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-		return -EINVAL;
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		break;
-	}
-
 	if (device->disconnected == disconnected)
 		return -EALREADY;
 
@@ -1316,6 +980,8 @@ const char *connman_device_get_string(struct connman_device *device,
 		return device->name;
 	else if (g_str_equal(key, "Node") == TRUE)
 		return device->node;
+	else if (g_str_equal(key, "Interface") == TRUE)
+		return device->interface;
 
 	return connman_element_get_string(&device->element, key);
 }
@@ -1333,6 +999,9 @@ static void set_offlinemode(struct connman_element *element, gpointer user_data)
 
 	device->offlinemode = offlinemode;
 
+	if (device->blocked == TRUE)
+		return;
+
 	powered = (offlinemode == TRUE) ? FALSE : TRUE;
 
 	if (device->powered == powered)
@@ -1358,12 +1027,21 @@ int __connman_device_set_offlinemode(connman_bool_t offlinemode)
 
 void __connman_device_increase_connections(struct connman_device *device)
 {
+	if (device == NULL)
+		return;
+
 	device->connections++;
 }
 
 void __connman_device_decrease_connections(struct connman_device *device)
 {
+	if (device == NULL)
+		return;
+
 	device->connections--;
+
+	if (device->connections == 0)
+		device->backoff_interval = SCAN_INITIAL_DELAY;
 }
 
 /**
@@ -1384,14 +1062,6 @@ int connman_device_add_network(struct connman_device *device,
 	if (identifier == NULL)
 		return -EINVAL;
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-		return -EINVAL;
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		break;
-	}
-
 	__connman_network_set_device(network, device);
 
 	err = connman_element_register((struct connman_element *) network,
@@ -1627,18 +1297,11 @@ static int device_load(struct connman_device *device)
 		device->powered_persistent = powered;
 	g_clear_error(&error);
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-		break;
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		val = g_key_file_get_integer(keyfile, identifier,
+	val = g_key_file_get_integer(keyfile, identifier,
 						"ScanInterval", &error);
-		if (error == NULL && val > 0)
-			device->scan_interval = val;
-		g_clear_error(&error);
-		break;
-	}
+	if (error == NULL)
+		device->scan_interval = val;
+	g_clear_error(&error);
 
 done:
 	g_free(identifier);
@@ -1667,16 +1330,8 @@ static int device_save(struct connman_device *device)
 	g_key_file_set_boolean(keyfile, identifier,
 					"Powered", device->powered_persistent);
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-		break;
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		if (device->scan_interval > 0)
-			g_key_file_set_integer(keyfile, identifier,
+	g_key_file_set_integer(keyfile, identifier,
 					"ScanInterval", device->scan_interval);
-		break;
-	}
 
 done:
 	g_free(identifier);
@@ -1697,8 +1352,6 @@ int __connman_device_init(void)
 {
 	DBG("");
 
-	connection = connman_dbus_get_connection();
-
 	if (connman_storage_register(&device_storage) < 0)
 		connman_error("Failed to register device storage");
 
@@ -1712,6 +1365,4 @@ void __connman_device_cleanup(void)
 	connman_driver_unregister(&device_driver);
 
 	connman_storage_unregister(&device_storage);
-
-	dbus_connection_unref(connection);
 }
