@@ -1365,6 +1365,8 @@ int __connman_ipconfig_address_add(struct connman_ipconfig *ipconfig)
 
 int __connman_ipconfig_address_remove(struct connman_ipconfig *ipconfig)
 {
+	int err;
+
 	DBG("");
 
 	if (ipconfig == NULL)
@@ -1381,13 +1383,19 @@ int __connman_ipconfig_address_remove(struct connman_ipconfig *ipconfig)
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4)
-			return connman_inet_clear_address(ipconfig->index,
+			err = connman_inet_clear_address(ipconfig->index,
 							ipconfig->address);
 		else if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6)
-			return connman_inet_clear_ipv6_address(
+			err = connman_inet_clear_ipv6_address(
 						ipconfig->index,
 						ipconfig->address->local,
 						ipconfig->address->prefixlen);
+		else
+			err = -EINVAL;
+
+		connman_ipaddress_clear(ipconfig->address);
+
+		return err;
 	}
 
 	return 0;
@@ -1693,7 +1701,8 @@ void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
 }
 
 void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
-							DBusMessageIter *iter)
+					DBusMessageIter *iter,
+					struct connman_ipconfig *ipconfig_ipv4)
 {
 	const char *str, *privacy;
 
@@ -1706,6 +1715,12 @@ void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
 	if (str == NULL)
 		return;
 
+	if (ipconfig_ipv4 != NULL &&
+			ipconfig->method == CONNMAN_IPCONFIG_METHOD_AUTO) {
+		if (__connman_6to4_check(ipconfig_ipv4) == 1)
+			str = "6to4";
+	}
+
 	connman_dbus_dict_append_basic(iter, "Method", DBUS_TYPE_STRING, &str);
 
 	if (ipconfig->system == NULL)
